#!/usr/bin/perl

# script to put an OpenTracker USB into KISS mode 
# Last update May 20, 2012

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# To obtain a copy of the GNU General Public License, write to
# the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.

my ($count, $result, $cmdflag);
my $kiss1 = "USB ON";

use strict;
use warnings;

use Device::SerialPort;

my $PortObj=Device::SerialPort->new("/dev/ttyACM0");

$PortObj->baudrate(57600);
$PortObj->parity("none");
$PortObj->databits(8);
$PortObj->stopbits(1);
$PortObj->handshake("rts");
$PortObj->write_settings;

$PortObj->read_char_time(0);
$PortObj->read_const_time(100);

# send some characters looking for a response
$PortObj->write("stuff\r");
$PortObj->write(chr(003));

$PortObj->read(2047); # read out lots of bytes in case lots of packets have come in
sleep (1);
# now send again and look for cmd prompt
$PortObj->write(chr(003));
$PortObj->write(chr(003));

# will iterate up to 4 times looking for the command prompt
# in case there's lots more in the buffer to read out
$count = 4; # choice of 4 is somewhat arbitrary
$cmdflag = 0;
while ($count > 0)  {
  $result = $PortObj->read(255);
  print "Reading Data...\n\n";
  if ($result =~ m/cmd:/)  {
    $cmdflag = 1;
    $count = 0;
  }
  else {
  $count = $count - 1;
  }
}

if ($cmdflag == 1)  {
  print "OK - got the cmd prompt\n\n";
}
else {
# retry getting the OT out of KISS MODE with control-C 's

  print "no command prompt - sending more control-C 's\n\n";
  sleep(1);
  $PortObj->write(chr(003));
  $PortObj->write(chr(003));
  $PortObj->write(chr(003));

#  check again and see if we can get command mode reponses
  $PortObj->write("junk\r");
  sleep(1);
  $PortObj->write("foo\r");
  $count = 4;
  while ($count > 0)  {
    $result = $PortObj->read(255);
    sleep(1);
    print "Reading data after sending commands to exit KISS mode...\n\n";
    print "$result\n\n";
    if ($result =~ m/cmd:/)  {
      $cmdflag = 2;
      $count = 0;
      print "OK, saw expected command prompt\n\n";
    }
    else {
    $count = $count - 1;
    }
  }
}

if ($cmdflag == 0)  {
  print "Was never able to communicate\n\n";
}

if ($cmdflag > 0)  {
  print "Sending commands to put the TNC in KISS mode\n\n";
# if we have a prompt, go ahead and put the TNC in KISS mode
  $PortObj->write($kiss1);
  sleep(1);
  $PortObj->write("\r");
  $result = $PortObj->read(255);
  print "$result\n\n";
  print "Command sent to put the TNC in KISS mode\n\n";
}
$PortObj->close;
